@@ -1,19 +1,31 @@
 import asyncio
+import sys
+from collections.abc import Awaitable, Collection, Coroutine, Iterator
+from contextlib import suppress
+from types import TracebackType
 from typing import (
     Any,
     Callable,
-    Collection,
-    Coroutine,
     Dict,
-    Iterator,
     Optional,
     Set,
+    Type,
     TypeVar,
+    Union,
 )
 
 from ._job import Job
 
+if sys.version_info >= (3, 11):
+    from asyncio import timeout as asyncio_timeout
+    from typing import Self
+else:
+    from async_timeout import timeout as asyncio_timeout
+
+    Self = TypeVar("Self", bound="Scheduler")
+
 _T = TypeVar("_T")
+_FutureLike = Union["asyncio.Future[_T]", Awaitable[_T]]
 ExceptionHandler = Callable[["Scheduler", Dict[str, Any]], None]
 
 
@@ -22,24 +34,28 @@ class Scheduler(Collection[Job[object]]):
         self,
         *,
         close_timeout: Optional[float] = 0.1,
+        wait_timeout: Optional[float] = 60,
         limit: Optional[int] = 100,
         pending_limit: int = 10000,
         exception_handler: Optional[ExceptionHandler] = None,
     ):
         if exception_handler is not None and not callable(exception_handler):
             raise TypeError(
-                "A callable object or None is expected, "
-                "got {!r}".format(exception_handler)
+                f"A callable object or None is expected, got {exception_handler!r}"
             )
 
         self._jobs: Set[Job[object]] = set()
+        self._shields: Set[asyncio.Task[object]] = set()
         self._close_timeout = close_timeout
+        self._wait_timeout = wait_timeout
         self._limit = limit
         self._exception_handler = exception_handler
-        self._failed_tasks: asyncio.Queue[
-            Optional[asyncio.Task[object]]
-        ] = asyncio.Queue()
-        self._failed_task = asyncio.create_task(self._wait_failed())
+        self._failed_tasks: asyncio.Queue[Optional[asyncio.Task[object]]] = (
+            asyncio.Queue()
+        )
+        self._failed_task: Optional[asyncio.Task[None]] = None
+        if sys.version_info < (3, 10):
+            self._failed_task = asyncio.create_task(self._wait_failed())
         self._pending: asyncio.Queue[Job[object]] = asyncio.Queue(maxsize=pending_limit)
         self._closed = False
 
@@ -61,6 +77,17 @@ class Scheduler(Collection[Job[object]]):
             state += " "
         return f"<Scheduler {state}jobs={len(self)}>"
 
+    async def __aenter__(self: Self) -> Self:
+        return self
+
+    async def __aexit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> None:
+        await self.wait_and_close()
+
     @property
     def limit(self) -> Optional[int]:
         return self._limit
@@ -90,6 +117,11 @@ class Scheduler(Collection[Job[object]]):
     ) -> Job[_T]:
         if self._closed:
             raise RuntimeError("Scheduling a new job after closing")
+        if self._failed_task is None:
+            self._failed_task = asyncio.create_task(self._wait_failed())
+        else:
+            if self._failed_task.get_loop() is not asyncio.get_running_loop():
+                raise RuntimeError(f"{self!r} is bound to a different event loop")
         job = Job(coro, self, name=name)
         should_start = self._limit is None or self.active_count < self._limit
         if should_start:
@@ -104,24 +136,80 @@ class Scheduler(Collection[Job[object]]):
         self._jobs.add(job)
         return job
 
+    def shield(self, arg: _FutureLike[_T]) -> "asyncio.Future[_T]":
+        inner = asyncio.ensure_future(arg)
+        if inner.done():
+            return inner
+
+        # This function is a copy of asyncio.shield(), except for the addition of
+        # the below 2 lines.
+        self._shields.add(inner)
+        inner.add_done_callback(self._shields.discard)
+
+        loop = inner.get_loop()
+        outer = loop.create_future()
+
+        def _inner_done_callback(inner: "asyncio.Task[object]") -> None:
+            if outer.cancelled():
+                if not inner.cancelled():
+                    inner.exception()
+                return
+
+            if inner.cancelled():
+                outer.cancel()
+            else:
+                exc = inner.exception()
+                if exc is not None:
+                    outer.set_exception(exc)
+                else:
+                    outer.set_result(inner.result())
+
+        def _outer_done_callback(outer: "asyncio.Future[object]") -> None:
+            if not inner.done():
+                inner.remove_done_callback(_inner_done_callback)
+
+        inner.add_done_callback(_inner_done_callback)
+        outer.add_done_callback(_outer_done_callback)
+        return outer
+
+    async def wait_and_close(self, timeout: Optional[float] = None) -> None:
+        if timeout is None:
+            timeout = self._wait_timeout
+        with suppress(asyncio.TimeoutError):
+            async with asyncio_timeout(timeout):
+                while self._jobs or self._shields:
+                    gather = asyncio.gather(
+                        *(job._wait() for job in self._jobs),
+                        *self._shields,
+                        return_exceptions=True,
+                    )
+                    await asyncio.shield(gather)
+        await self.close()
+
     async def close(self) -> None:
         if self._closed:
             return
         self._closed = True  # prevent adding new jobs
 
         jobs = self._jobs
-        if jobs:
+        if jobs or self._shields:
             # cleanup pending queue
             # all job will be started on closing
             while not self._pending.empty():
                 self._pending.get_nowait()
+
+            for f in self._shields:
+                f.cancel()
+
             await asyncio.gather(
-                *[job._close(self._close_timeout) for job in jobs],
+                *(job._close(self._close_timeout) for job in jobs),
+                *(asyncio.wait_for(f, self._close_timeout) for f in self._shields),
                 return_exceptions=True,
             )
             self._jobs.clear()
-        self._failed_tasks.put_nowait(None)
-        await self._failed_task
+        if self._failed_task is not None:
+            self._failed_tasks.put_nowait(None)
+            await self._failed_task
 
     def call_exception_handler(self, context: Dict[str, Any]) -> None:
         if self._exception_handler is None:
